@@ -18,41 +18,74 @@ import urllib2
 import re
 import time
 from os import path
+from calculate.lib.configparser import ConfigParserCaseSens, Error as CPError
 from calculate.lib.utils.tools import SingletonParam
 
 
 class BinhostsBase(object):
-    def __init__(self, timeout, revision_path, ts_path, last_ts, binhost_list):
+    class BinhostStatus:
+        Success = 0
+        EnvNotFound = 1
+        Updating = 2
+        BadEnv = 3
+        UnknownError = 4
+
+    def __init__(self, timeout, revision_path, ts_path, last_ts, binhost_list,
+                 arch=None, skip_timestamp=False):
         self.timeout = int(timeout)
         self.revision_path = revision_path
         self.ts_path = ts_path
         self.last_ts = last_ts
         self.binhost_list = binhost_list
         self.data = None
+        self.arch = arch
+        self.skip_timestamp = skip_timestamp
         self.binhosts_data = {}
 
+    def check_package_timestamp(self, fn, timestamp):
+        for i, line in enumerate(
+                urllib2.urlopen(fn, timeout=self.timeout)):
+            if line.startswith("TIMESTAMP"):
+                return str(timestamp) == line.rpartition(":")[2].strip()
+            if i > 50:
+                break
+        return False
+
     def check_binhost(self, binhost):
+        status = self.BinhostStatus.Success
         if binhost not in self.binhosts_data:
             revision_files = [path.join(binhost, x)
                               for x in self.revision_path]
+            data = ""
             try:
-                data = None
                 for fn in revision_files:
-                    if data is None:
-                        data = urllib2.urlopen(fn, timeout=self.timeout).read()
-                    elif data != urllib2.urlopen(fn,
-                                                 timeout=self.timeout).read():
-                        res = ""
-                        break
-                else:
-                    res = data
-            except urllib2.URLError:
-                res = ""
+                    base_dn = path.dirname(fn)
+                    data = urllib2.urlopen(fn, timeout=self.timeout).read()
+                    cp = ConfigParserCaseSens()
+                    if data:
+                        try:
+                            cp.read_string(unicode(data))
+                            for pkg_file in cp['timestamp']:
+                                if not self.check_package_timestamp(
+                                        path.join(base_dn, pkg_file),
+                                        cp['timestamp'][pkg_file]):
+                                    status = self.BinhostStatus.Updating
+                                    data = ""
+                                    break
+                        except (CPError, KeyError):
+                            status = self.BinhostStatus.BadEnv
+                            data = ""
+                            break
+                    break
+            except urllib2.URLError as e:
+                status = self.BinhostStatus.EnvNotFound
+                pass
             except BaseException as e:
                 if isinstance(e, KeyboardInterrupt):
                     raise
-                res = ""
-            self.binhosts_data[binhost] = res
+                status = self.BinhostStatus.UnknownError
+                data = ""
+            self.binhosts_data[binhost] = status, data
         return self.binhosts_data[binhost]
 
     re_revison = re.compile("\w+=(\w+)")
@@ -69,7 +102,7 @@ class BinhostsBase(object):
                                    timeout=self.timeout).read().strip()
             if data.isdigit() and self.last_ts.isdigit():
                 return (data, int((time.time() - t) * 1000),
-                        t - int(data) < 5 * DAY,
+                        t - int(data) < 7 * DAY,
                         int(data) < int(self.last_ts))
         except urllib2.URLError as e:
             pass
